@@ -28,21 +28,6 @@
 #include "cd-recorder.h"
 #include "cd-recorder-marshal.h"
 
-typedef enum {
-	PREPARING_TO_WRITE_CD,
-	WRITING_CD,
-	FIXATING_CD,
-	BLANKING_CD,
-} ActionType;
-
-static char *actions [] = {
-	N_("Preparing to write CD"),
-	N_("Writing CD"),
-	N_("Fixating CD"),
-	N_("Erasing CD"),
-	NULL
-};
-
 struct CDRecorderPrivate {
 	GMainLoop *loop;
 	int result;
@@ -52,7 +37,6 @@ struct CDRecorderPrivate {
 	GString *cdr_stderr;
 	gboolean changed_text;
 	gboolean expect_cdrecord_to_die;
-	GList *actions;
 	gboolean dangerous;
 
 	char *last_error;
@@ -165,6 +149,25 @@ cd_recorder_cancel (CDRecorder *cdrecorder, gboolean skip_if_dangerous)
 	return cdrecorder->priv->dangerous;
 }
 
+static void
+insert_cd_retry (CDRecorder *cdrecorder, gboolean cancel,
+		 gboolean is_reload, gboolean send_return)
+{
+	if (cancel) {
+		/* Shouldn't be dangerous on reload */
+		cd_recorder_cancel (cdrecorder, FALSE);
+	} else if (is_reload) {
+		if (send_return) {
+			write (cdrecorder->priv->cdr_stdin, "\n", 1);
+		} else {
+			kill (cdrecorder->priv->pid, SIGUSR1);
+		}
+	} else {
+		cdrecorder->priv->result = RESULT_RETRY;
+		g_main_loop_quit (cdrecorder->priv->loop);
+	}
+}
+
 static gboolean
 growisofs_stdout_read (GIOChannel   *source,
 		      GIOCondition  condition,
@@ -198,7 +201,7 @@ growisofs_stdout_read (GIOChannel   *source,
 			if (!cdrecorder->priv->changed_text) {
 				g_signal_emit (G_OBJECT (cdrecorder),
 					       cd_recorder_table_signals[ACTION_CHANGED], 0,
-					       WRITING_CD);
+					       WRITING, MEDIA_DVD);
 			}
 
 			percent = (perc_1 + ((float) perc_2 / 10)) / 100;
@@ -237,6 +240,7 @@ cdrecord_stdout_read (GIOChannel   *source,
 	char buf[1];
 	unsigned int track, mb_written, mb_total;
 	GIOStatus status;
+	gboolean res;
 
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
@@ -259,7 +263,7 @@ cdrecord_stdout_read (GIOChannel   *source,
 			if (!cdrecorder->priv->changed_text) {
 				g_signal_emit (G_OBJECT (cdrecorder),
 					       cd_recorder_table_signals[ACTION_CHANGED], 0,
-					       WRITING_CD);
+					       WRITING, MEDIA_CD);
 			}
 			percent = 0.98 * ((double)((track-1)/(double)cdrecorder->priv->track_count)
 					  + ((double)mb_written/mb_total) / (double)cdrecorder->priv->track_count);
@@ -268,11 +272,19 @@ cdrecord_stdout_read (GIOChannel   *source,
 				       percent);
 		} else if (g_str_has_prefix (line, "Re-load disk and hit <CR>") ||
 			   g_str_has_prefix (line, "send SIGUSR1 to continue")) {
-			g_assert_not_reached ();
+			/* This is not supposed to happen since we checked for the cd
+			   before starting, but we try to handle it anyway, since mmc
+			   profiling can fail. */
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[INSERT_CD_REQUEST],
+				       0, TRUE, cdrecorder->priv->can_rewrite,
+				       FALSE, &res);
+			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+			insert_cd_retry (cdrecorder, !res, TRUE, (*line == 'R'));
 		} else if (g_str_has_prefix (line, "Fixating...")) {
 			g_signal_emit (G_OBJECT (cdrecorder),
 				       cd_recorder_table_signals[ACTION_CHANGED], 0,
-				       FIXATING_CD);
+				       FIXATING, MEDIA_CD);
 		} else if (g_str_has_prefix (line, "Fixating time:")) {
 			g_signal_emit (G_OBJECT (cdrecorder),
 				       cd_recorder_table_signals[PROGRESS_CHANGED], 0,
@@ -283,7 +295,7 @@ cdrecord_stdout_read (GIOChannel   *source,
 		} else if (g_str_has_prefix (line, "Blanking PMA, TOC, pregap")) {
 			g_signal_emit (G_OBJECT (cdrecorder),
 				       cd_recorder_table_signals[ACTION_CHANGED], 0,
-				       BLANKING_CD);
+				       BLANKING, MEDIA_CD);
 		}
 
 		g_free (line);
@@ -312,6 +324,7 @@ growisofs_stderr_read (GIOChannel   *source,
 	CDRecorder *cdrecorder = (CDRecorder *) data;
 	char *line;
 	GIOStatus status;
+	gboolean res;
 
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
@@ -323,7 +336,15 @@ growisofs_stderr_read (GIOChannel   *source,
 	if (status == G_IO_STATUS_NORMAL && !cdrecorder->priv->expect_cdrecord_to_die) {
 		g_string_prepend (cdrecorder->priv->cdr_stderr, line);
 		if (strstr (line, "unsupported MMC profile") != NULL || (strstr (line, "already carries isofs") != NULL && strstr (line, "FATAL:") != NULL)) {
-			g_assert_not_reached ();
+			/* This is not supposed to happen since we checked for the cd
+			   type before starting, but we try to handle it anyway, since mmc
+			   profiling can fail. */
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[INSERT_CD_REQUEST],
+				       0, TRUE, cdrecorder->priv->can_rewrite,
+				       FALSE, &res);
+			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+			insert_cd_retry (cdrecorder, !res, FALSE, FALSE);
 		} else if (strstr (line, "unable to open") != NULL || strstr (line, "unable to stat") != NULL) {
 			/* This fits the "open64" and "open"-like messages */
 			cdrecorder->priv->last_error = g_strdup (_("The recorder could not be accessed"));
@@ -340,7 +361,7 @@ growisofs_stderr_read (GIOChannel   *source,
 		} else if (strstr (line, "flushing cache") != NULL) {
 			g_signal_emit (G_OBJECT (cdrecorder),
 				       cd_recorder_table_signals[ACTION_CHANGED], 0,
-				       FIXATING_CD);
+				       FIXATING, MEDIA_DVD);
 			cdrecorder->priv->result = RESULT_FINISHED;
 		} else if (strstr (line, ":-(") != NULL || strstr (line, "FATAL") != NULL) {
 			cdrecorder->priv->last_error = g_strdup (_("Unhandled error, aborting"));
@@ -352,7 +373,7 @@ growisofs_stderr_read (GIOChannel   *source,
 		}
 		return FALSE;
 	} else {
-		g_print ("cdrecord stderr read failed, status: %d\n", status);
+		g_print ("growisofs stderr read failed, status: %d\n", status);
 	}
 
 	return TRUE;
@@ -366,6 +387,7 @@ cdrecord_stderr_read (GIOChannel   *source,
 	CDRecorder *cdrecorder = (CDRecorder *) data;
 	char *line;
 	GIOStatus status;
+	gboolean res;
 
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
@@ -377,14 +399,28 @@ cdrecord_stderr_read (GIOChannel   *source,
 	if (status == G_IO_STATUS_NORMAL && !cdrecorder->priv->expect_cdrecord_to_die) {
 		g_string_prepend (cdrecorder->priv->cdr_stderr, line);
 		if (strstr (line, "No disk / Wrong disk!") != NULL) {
-			g_warning ("No disk in drive, shouldn't happen");
-			g_assert_not_reached ();
+			/* This is not supposed to happen since we checked for the cd
+			   before starting, but we try to handle it anyway, since mmc
+			   profiling can fail. */
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[INSERT_CD_REQUEST],
+				       0, TRUE, cdrecorder->priv->can_rewrite,
+				       FALSE, &res);
+			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+			insert_cd_retry (cdrecorder, !res, FALSE, FALSE);
 		} else if (strstr (line, "This means that we are checking recorded media.") != NULL) {
 			cdrecorder->priv->last_error = g_strdup (_("The CD has already been recorded"));
 			cdrecorder->priv->result = RESULT_ERROR;
 		} else if (strstr (line, "Cannot blank disk, aborting.") != NULL) {
-			g_warning ("Right CD type wasn't loaded, shouldn't happen");
-			g_assert_not_reached ();
+			/* This is not supposed to happen since we checked for the cd
+			   type before starting, but we try to handle it anyway, since
+			   mmc profiling can fail. */
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[INSERT_CD_REQUEST],
+				       0, TRUE, TRUE,
+				       FALSE, &res);
+			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+			insert_cd_retry (cdrecorder, !res, FALSE, FALSE);
 		} else if (strstr (line, "Data may not fit on current disk") != NULL) {
 			cdrecorder->priv->last_error = g_strdup (_("The files selected did not fit on the CD"));
 			/* FIXME should we error out in that case?
@@ -393,8 +429,17 @@ cdrecord_stderr_read (GIOChannel   *source,
 			cdrecorder->priv->last_error = g_strdup (_("All audio files must be stereo, 16-bit digital audio with 44100Hz samples"));
 			cdrecorder->priv->result = RESULT_ERROR;
 		} else if (strstr (line, "cannot write medium - incompatible format") != NULL) {
-			g_assert_not_reached ();
+			/* This is not supposed to happen since we checked for the cd
+			   type before starting, but we try to handle it anyway, since
+			   mmc profiling can fail. */
+			g_signal_emit (G_OBJECT (cdrecorder),
+				       cd_recorder_table_signals[INSERT_CD_REQUEST],
+				       0, TRUE, cdrecorder->priv->can_rewrite,
+				       FALSE, &res);
 			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+			insert_cd_retry (cdrecorder, !res, FALSE, FALSE);
+		} else if (strstr (line, "DMA speed too slow") != NULL) {
+			cdrecorder->priv->last_error = g_strdup (_("The system is too slow to write the CD at this speed. Try a lower speed."));
 		}
 
 		g_free (line);
@@ -411,10 +456,8 @@ cdrecord_stderr_read (GIOChannel   *source,
 }
 
 static gboolean
-media_type_matches (CDMediaType type, gboolean *needs_blank)
+media_type_matches (CDMediaType type)
 {
-	*needs_blank = FALSE;
-
 	switch (type)
 	{
 	case CD_MEDIA_TYPE_ERROR:
@@ -429,17 +472,14 @@ media_type_matches (CDMediaType type, gboolean *needs_blank)
 	case CD_MEDIA_TYPE_CDR:
 		return TRUE;
 	case CD_MEDIA_TYPE_CDRW:
-		*needs_blank = TRUE;
 		return TRUE;
 	case CD_MEDIA_TYPE_DVDR:
 		return TRUE;
 	case CD_MEDIA_TYPE_DVDRW:
-		*needs_blank = TRUE;
 		return TRUE;
 	case CD_MEDIA_TYPE_DVD_PLUS_R:
 		return TRUE;
 	case CD_MEDIA_TYPE_DVD_PLUS_RW:
-		*needs_blank = TRUE;
 		return TRUE;
 	}
 
@@ -448,19 +488,19 @@ media_type_matches (CDMediaType type, gboolean *needs_blank)
 
 static CDMediaType
 cd_recorder_wait_for_insertion (CDRecorder *cdrecorder, CDDrive *drive,
-		gboolean *needs_blank)
+				gboolean *needs_blank)
 {
 	CDMediaType type;
 	gboolean reload;
 
 	reload = FALSE;
-	type = cd_drive_get_media_type (drive);
+	type = cd_drive_get_media_type_and_rewritable (drive, needs_blank);
 
 	if (type == CD_MEDIA_TYPE_ERROR) {
 		reload = TRUE;
 	}
 
-	while (!media_type_matches (type, needs_blank)) {
+	while (!media_type_matches (type)) {
 		gboolean res, busy_cd;
 
 		busy_cd = (type == CD_MEDIA_TYPE_BUSY);
@@ -481,7 +521,7 @@ cd_recorder_wait_for_insertion (CDRecorder *cdrecorder, CDDrive *drive,
 			return CD_MEDIA_TYPE_ERROR;
 		}
 
-		type = cd_drive_get_media_type (drive);
+		type = cd_drive_get_media_type_and_rewritable (drive, needs_blank);
 		reload = FALSE;
 		if (type == CD_MEDIA_TYPE_UNKNOWN
 				|| type == CD_MEDIA_TYPE_ERROR) {
@@ -502,6 +542,7 @@ cd_recorder_write_tracks (CDRecorder *cdrecorder,
 {
 	CDMediaType type;
 	gboolean needs_blank;
+	gboolean is_locked;
 
 	g_return_val_if_fail (tracks != NULL, RESULT_ERROR);
 	cdrecorder->priv->tracks = tracks;
@@ -518,8 +559,10 @@ cd_recorder_write_tracks (CDRecorder *cdrecorder,
 		return cdrecorder->priv->result;
 	}
 
+	is_locked = cd_drive_lock (drive, _("Burning CD"), NULL);
+
 	type = cd_recorder_wait_for_insertion (cdrecorder,
-			drive, &needs_blank);
+					       drive, &needs_blank);
 
 	if (type == CD_MEDIA_TYPE_ERROR) {
 		cdrecorder->priv->result = RESULT_CANCEL;
@@ -539,6 +582,9 @@ cd_recorder_write_tracks (CDRecorder *cdrecorder,
 						   drive, tracks, speed,
 						   flags);
 	}
+	if (is_locked) {
+		cd_drive_unlock (drive);
+	}
 }
 
 static int
@@ -602,7 +648,7 @@ retry_growisofs:
 
 	g_signal_emit (G_OBJECT (cdrecorder),
 			cd_recorder_table_signals[ACTION_CHANGED], 0,
-			PREPARING_TO_WRITE_CD);
+			PREPARING_WRITE, MEDIA_DVD);
 	cdrecorder->priv->changed_text = FALSE;
 	g_signal_emit (G_OBJECT (cdrecorder),
 			cd_recorder_table_signals[PROGRESS_CHANGED], 0,
@@ -736,6 +782,9 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 	if (flags & CDRECORDER_OVERBURN) {
 		g_ptr_array_add (argv, "-overburn");
 	}
+	if (flags & CDRECORDER_BURNPROOF) {
+		g_ptr_array_add (argv, "driveropts=burnfree");
+	}
 	g_ptr_array_add (argv, "-v");
 
 	l = tracks;
@@ -774,7 +823,7 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 
 	g_signal_emit (G_OBJECT (cdrecorder),
 		       cd_recorder_table_signals[ACTION_CHANGED], 0,
-		       PREPARING_TO_WRITE_CD);
+		       PREPARING_WRITE, MEDIA_CD);
 	cdrecorder->priv->changed_text = FALSE;
 	g_signal_emit (G_OBJECT (cdrecorder),
 		       cd_recorder_table_signals[PROGRESS_CHANGED], 0,
@@ -854,12 +903,6 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 	return cdrecorder->priv->result;
 }
 
-GList *
-cd_recorder_list_actions (CDRecorder *cdrecorder)
-{
-	return cdrecorder->priv->actions;
-}
-
 const char *
 cd_recorder_get_error_message_details (CDRecorder *cdrecorder)
 {
@@ -892,7 +935,6 @@ cd_recorder_finalize (GObject *object)
 	}
 
 	g_free (cdrecorder->priv->last_error);
-	g_list_free (cdrecorder->priv->actions);
 
 	if (G_OBJECT_CLASS (parent_class)->finalize != NULL) {
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
@@ -902,16 +944,7 @@ cd_recorder_finalize (GObject *object)
 static void
 cd_recorder_init (CDRecorder *cdrecorder)
 {
-	int i;
-
 	cdrecorder->priv = g_new0 (CDRecorderPrivate, 1);
-
-	for (i = 0; actions[i] != NULL; i++)
-	{
-		cdrecorder->priv->actions = g_list_append
-			(cdrecorder->priv->actions,
-			 _(actions[i]));
-	}
 }
 
 CDRecorder *
@@ -944,8 +977,8 @@ cd_recorder_class_init (CDRecorderClass *klass)
 				G_STRUCT_OFFSET (CDRecorderClass,
 					action_changed),
 				NULL, NULL,
-				g_cclosure_marshal_VOID__INT,
-				G_TYPE_NONE, 1, G_TYPE_INT);
+				cdrecorder_marshal_VOID__INT_INT,
+				G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
 	cd_recorder_table_signals[ANIMATION_CHANGED] =
 		g_signal_new ("animation-changed",
 				G_TYPE_FROM_CLASS (klass),
