@@ -39,7 +39,9 @@ typedef enum {
 	CDRECORDER_BLANK			= 1 << 1,
 	CDRECORDER_DUMMY_WRITE			= 1 << 2,
 	CDRECORDER_DISC_AT_ONCE			= 1 << 3,
-	CDRECORDER_DEBUG			= 1 << 4
+	CDRECORDER_DEBUG			= 1 << 4,
+	CDRECORDER_OVERBURN			= 1 << 5,
+	CDRECORDER_BURNPROOF			= 1 << 6
 } CDRecorderWriteFlags;
 
 #define CD_TYPE_RECORDER            (cd_recorder_get_type ())
@@ -52,6 +54,18 @@ typedef struct CDRecorder                CDRecorder;
 typedef struct CDRecorderClass           CDRecorderClass;
 typedef struct CDRecorderPrivate         CDRecorderPrivate;
 
+typedef enum {
+	PREPARING_WRITE,
+	WRITING,
+	FIXATING,
+	BLANKING
+} CDRecorderActions;
+
+typedef enum {
+	MEDIA_CD,
+	MEDIA_DVD,
+} CDRecorderMedia;
+
 struct CDRecorder {
 	GObject parent;
 	CDRecorderPrivate *priv;
@@ -63,22 +77,21 @@ struct CDRecorderClass {
 	void (*progress_changed)	(CDRecorder *cdrecorder,
 					 gdouble fraction);
 	void (*action_changed)		(CDRecorder *cdrecorder,
-					 gint index);
+					 CDRecorderActions action,
+					 CDRecorderMedia media);
 	void (*animation_changed)	(CDRecorder *cdrecorder,
 					 gboolean spinning);
-	void (*insert_cd_request)	(CDRecorder *cdrecorder,
-					 gboolean is_reload);
-	void (*insert_cd_rw_request)	(CDRecorder *cdrecorder,
-					 gboolean is_reload);
+	gboolean (*insert_cd_request)	(CDRecorder *cdrecorder,
+					 gboolean is_reload,
+					 gboolean can_rewrite,
+					 gboolean busy_cd);
 };
 
 GType		cd_recorder_get_type		(void);
 CDRecorder     *cd_recorder_new			(void);
 
-GList	       *cd_recorder_list_actions	(CDRecorder *cdrecorder);
-
 int		cd_recorder_write_tracks	(CDRecorder *cdrecorder,
-						 CDDrive *recorder,
+						 CDDrive *drive,
 						 GList *tracks,
 						 gint speed,
 						 CDRecorderWriteFlags flags);
@@ -89,8 +102,6 @@ gboolean	cd_recorder_cancel		(CDRecorder *cdrecorder,
 const char*	cd_recorder_get_error_message	(CDRecorder *cdrecorder);
 const char*	cd_recorder_get_error_message_details
 						(CDRecorder *cdrecorder);
-void		cd_recorder_insert_cd_retry	(CDRecorder *cdrecorder,
-						 gboolean is_reload);
 void		cd_recorder_track_free		(Track *track);
 
 #endif /* CD_RECORDER_H */
